// Quad_ROV_Joystick_PID_ECHO
//
// Written by Theron Wierenga, December 2015, updated February 2017
// Modified September 18, 2017
//
// Defined commands sent to ROV
//
// R and int from -15 to 15 degrees   Roll
// P and int from -15 to 15 degrees   Pitch
// Y and int from -10 to 10 degrees   Yaw
// D and int from -1 to 1 inches      Depth (Thrust)
//
// Defined data echoed back to joystick controller
//
// Joystick Output
//
//       1023
//
//
// 0              1023
//
//
//         0
//
// Right LARGE Joystick 
// X = A3  Roll   Left/Right movement
// Y = A2  Pitch  Forward/Back movement
// Swirch = D3
//
// Left LARGE Joystick
// X = A6  Yaw    Left/Right
// Y = A7  Thrust Forward/Back
// Switch = D2
//
// Right Small Joystick 
// X = A2  Roll   Left/Right movement
// Y = A3  Pitch  Forward/Back movement
// Swirch = D3
//
// Left Small Joystick
// X = A7  Yaw    Left/Right
// Y = A6  Thrust Forward/Back
// Switch = D2
//
// Battery level is read on A0
//
// Serial to Quad_ROV Teensy 3.1 is to D9, D10, RX, TX
// Serial for Joysticks is D4, D5, RX, TX

#define SWITCH0 3 // Right
#define SWITCH1 2 // Left
#define deadzone 15

#include <Wire.h>
#include <LiquidCrystal_I2C.h>
// Some display have ab address of 0x27, this one is 0x3F
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address
#include <SoftwareSerial.h>
//SoftwareSerial mySerial(4, 5); // RX, TX, this for large joysticks
SoftwareSerial mySerial(10, 11); // RX, TX, this for small joysticks

String myInputString = "", inputString = "", strTemp = "", numTemp = "";
boolean myStringComplete = false, stringComplete = false, debug = false;

int xRight, xLeft, yRight, yLeft, depthChange, s0, s1, change = 1;
int roll, pitch, yaw, depth, lastRoll = 0, lastPitch = 0, lastYaw = 0, lastDepth = 0,lasts0 = 1, lasts1 = 1;
unsigned long time_now, time_last;

void setup()
{
  pinMode(SWITCH0, INPUT_PULLUP);
  pinMode(SWITCH1, INPUT_PULLUP);
  // Clicking right or left switch pulls input to ground
  Serial.begin(115200);
  // reserve 200 bytes for the inputString:
  inputString.reserve(200);
  mySerial.begin(19200);
  myInputString.reserve(200);
  // Start the LCD display
  lcd.begin(20, 4);
  lcd.setCursor(0, 3);
  lcd.print("Start");
  time_last = millis() / 500;  // Send every 1/2 second for depth
  
}

void loop()                    
{
    //                                          Roll
    xRight = analogRead(A2);
    roll = normalizeR(float(xRight));
    if (abs(roll - lastRoll) >= change)      // If changed send to controller
    {
        strTemp = "R" + String(roll);
        mySerial.println(strTemp);
        Serial.println(strTemp);
        lastRoll = roll;
        delay(50);
    }

    //                                          Pitch
    yRight = analogRead(A3); 
    pitch = normalizeP(float(yRight));
    if (abs(pitch - lastPitch) >= change)   // If changed send to controller
    {
        strTemp = "P" + String(pitch);
        mySerial.println(strTemp);
        Serial.println(strTemp);
        lastPitch = pitch;
        delay(50);
    }

    //                                          Yaw
    xLeft = analogRead(A7);
    yaw = normalizeYaw(float(xLeft));
    if (abs(yaw - lastYaw) >= change)     // If changed send to controller
    {
        strTemp = "Y" + String(yaw);
        mySerial.println(strTemp);
        Serial.println(strTemp);
        lastYaw = yaw;
        delay(50);
    }

    //                                          Depth
    yLeft = analogRead(A6);
    depth = normalizeDepth(float(yLeft));
    if (depth != 0)
    {
      time_now = millis() / 500;
      if (time_now - time_last >= 1)               // If 1/2 second has passed send depth change
      {
        strTemp = "D" + String(depth);
        mySerial.println(strTemp);
        Serial.println(strTemp);
        time_last = millis() / 500;
        delay(50);
      }
    }

    // Switches are currently not set up to be reviewed by controller
    //                                          Switch 0, on right
    s0 = digitalRead(SWITCH0);
    if (s0 != lasts0)
    {
      if (s0 == 0)
      {
          // Right switch pressed
          mySerial.println("S0");
          Serial.println("S0");
      }
      lasts0 = s0;
      delay(50);
    }

    //                                          Switch 1, on left
    s1 = digitalRead(SWITCH1);
    if (s1 != lasts1)
    {
      if (s1 == 0)
      {
          // Left switch pressed
          mySerial.println("S1");
          Serial.println("S1");
      }
      lasts1 = s1;
      delay(50);
    }

    //  Check if anything received from controller below
    // R = roll
    // P = pitch
    // Y = yaw
    // D = depth in inches
    // G = depthInput
    // T = depthSetpoint
    // S = Kp value
    mySerialEvent();
    if (myStringComplete)     // Something is received so display it on the LCD
    {
        Serial.println(myInputString);
        strTemp = myInputString.substring(0,1);
       // Serial.println(strTemp);
        numTemp = myInputString.substring(1, myInputString.length() - 1);
       // Serial.println(numTemp);
       // Serial.println(numTemp.length());

        if (strTemp == "R")
        {
          lcd.setCursor(0, 0);
          lcd.print("     ");
          lcd.setCursor(0, 0);
          lcd.print("R ");
          lcd.print(numTemp);
        }
        if (strTemp == "P")
        {
          lcd.setCursor(0, 1);
          lcd.print("     ");
          lcd.setCursor(0, 1);
          lcd.print("P ");
          lcd.print(numTemp);
        }
        if (strTemp == "Y")
        {
          lcd.setCursor(0, 2);
          lcd.print("     ");
          lcd.setCursor(0, 2);
          lcd.print("Y ");
          lcd.print(numTemp);
        }
        if (strTemp == "D")
        {
          lcd.setCursor(0, 3);
          lcd.print("        ");
          lcd.setCursor(0, 3);
          lcd.print("D ");
          lcd.print(numTemp);
        }
        if (strTemp == "S")
        {
          lcd.setCursor(10, 0);
          lcd.print("        ");
          lcd.setCursor(10, 0);
          lcd.print("S ");
          lcd.print(numTemp);  
        }
        if (strTemp == "G")
        {
          lcd.setCursor(10, 1);
          lcd.print("        ");
          lcd.setCursor(10, 1);
          lcd.print("G ");
          lcd.print(numTemp);
        }
        if (strTemp == "T")
        {
          lcd.setCursor(10, 2);
          lcd.print("        ");
          lcd.setCursor(10, 2);
          lcd.print("T ");
          lcd.print(numTemp);
        }
        if (strTemp == "A") // Actual depth
        {
          lcd.setCursor(10, 3);
          lcd.print("        ");
          lcd.setCursor(10, 3);
          lcd.print("A ");
          lcd.print(numTemp);
        }
        myStringComplete = false;
        myInputString = "";
    }
 //   delay(100);
}

// For receiving characters
void mySerialEvent()
{
  while ((mySerial.available() > 0) && (myStringComplete == false))
  {
    // get the new byte:
    char inChar = (char)mySerial.read();
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n')
    {
      myStringComplete = true;
      break;
    }
    // add it to the inputString:
    myInputString += inChar;
    delay(2);
  }
}

// Takes in 0 to 1023
// Returns  15 to -15 as int
int normalizeP(float x)
{
  float temp = (((1023.0 - x) / 1023.0) * 30.0) - 15.0 ;
  if ((x > (512 - deadzone)) && (x < (512 + deadzone))) temp = 0.0; 
  if (temp < 0.0)
  {
    return int(temp - 0.5);
  }
  else
  {
    return int(temp + 0.5);
  }
}

// Takes in 0 to 1023
// Returns -15 to 15 as int
int normalizeR(float x)
{
  float temp = ((x / 1023.0) * 30.0) - 15.0 ;
  if ((x > (512 - deadzone)) && (x < (512 + deadzone))) temp = 0.0; 
  if (temp < 0.0)
  {
    return int(temp - 0.5);
  }
  else
  {
    return int(temp + 0.5);
  }
}

// Takes in 0 to 1023
// Returns -10 to 10 as int
int normalizeYaw(float x)
{
  float temp = (((1023.0 - x) / 1023.0) * 20.0) - 10.0 ;
  if ((x > (512 - deadzone)) && (x < (512 + deadzone))) temp = 0.0; 
  if (temp < 0.0)
  {
    return int(temp - 0.5);
  }
  else
  {
    return int(temp + 0.5);
  }
}

// Takes in 0 to 1023
// Returns - 1 to 1 as int
int normalizeDepth(float x)
{
  int temp;
  if (x < 512) {temp = -1;} else {temp = 1;}
  if ((x > (512 - deadzone)) && (x < (512 + deadzone))) temp = 0; 
  //Serial.println(temp);
  return temp;
}


